/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla addressbook.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * scripts.uiscripts.Main
 *
 * $Id: main.js,v 1.4 2003/03/31 11:44:11 prefiks Exp $
 *
 */
ML.load("uiscripts/alertmgr.js");

function MainHandler()
{
    this.bundle = getStringBundle("chrome://jim/locale/jim.properties");
}

MainHandler.prototype =
{
    openXMLConsole: function()
    {
        var win = this.__getWindow("jim:xml-logger");
        if (win)
            win.focus();
        else
            open("chrome://jim/content/xmllogger.xul",
                "_blank", "chrome,resizable=yes");
    },

    openCommandConsole: function()
    {
        var win = this.__getWindow("jim:command-console");
        if (win)
            win.focus();
        else
            open("chrome://jim/content/command.xul",
                "_blank", "chrome,resizable=yes");
    },

    editAccount: function()
    {
        var el = document.getElementById("profile-name").currentItem;

        if (el != null) {
            var win = this.__getWindow("jim:edit-account");
            if (win)
                win.focus();
            else
                openDialog("chrome://jim/content/editAccount.xul",
                    "_blank", "chrome,all,dialog=yes,modal",
                    el.getAttribute("label"));
        }
    },

    addAccount: function()
    {
        var win = this.__getWindow("jim:add-account");
        if (win)
            win.focus();
        else
            openDialog("chrome://jim/content/addAccount.xul",
                "_blank", "chrome,all,dialog=yes,modal");
    },

    addContact: function()
    {
        var win = this.__getWindow("jim:add-contact");
        if (win)
            win.focus();
        else
            openDialog("chrome://jim/content/addContact.xul",
                "_blank", "chrome,all,dialog=yes,modal");
    },

    removeAccount: function()
    {
        var el = document.getElementById("profile-name").currentItem;

        if (el != null) {
            var win = this.__getWindow("jim:remove-account");
            if (win)
                win.focus();
            else
                openDialog("chrome://jim/content/removeAccount.xul",
                    "_blank", "chrome,all,dialog=yes,modal",
                    el.getAttribute("label"));
        }
    },

    profileChanged: function()
    {
        var el = document.getElementById("profile-name").currentItem, v;
        var info;
            
        if (!el)
            return;
        info = jabberService.core.highlevel.accounts.getAccountInfo(
            el.getAttribute("label"));

        if (info == null || info.password != null)
            document.getElementById("password-box").setAttribute("style",
                "visibility: hidden;");
        else
            document.getElementById("password-box").setAttribute("style",
                "visibility: visible;");
    },

    showAll: function(from)
    {
        var el = document.getElementById("commands.menu.jim.showAll");
        var v = el.getAttribute("checked") == "false";
        
        el.setAttribute("checked", v ? "true" : "false");
        document.getElementById('user-list').showAll = v;
    },

    setShow: function(show)
    {
        jabberService.core.highlevel.users.setPresence(null, show);
    },

    setType: function(type)
    {
        jabberService.core.highlevel.users.setPresence(type);
    },

    registerAgent: function(el)
    {
        var jid = el.getAttribute("jid");
        alert("reg "+jid);
    },

    searchAgent: function(el)
    {
        var jid = el.getAttribute("jid");
        alert("search "+jid);
    },

    doConnect: function()
    {
        var el = document.getElementById("profile-name").currentItem, v;
        var password = document.getElementById('password').value;
            
        if (!el)
            return;

        info = jabberService.core.highlevel.accounts.getAccountInfo(
            el.getAttribute("label"));

        if (!info)
            return;

        jabberService.connect(info.server, info.port, info.useSSL);
        jabberService.core.iq.auth.authenticate(info.username, info.password ||
            password, info.resource);
        document.getElementById("main-deck").selectedIndex = 2;
    },

    onLoad: function()
    {
        var el = document.getElementById("profile-name");
        el.database.AddDataSource(jabberService.rdf.DS);
        el.builder.rebuild();

        el = document.getElementById("services-to-register");
        el.database.AddDataSource(jabberService.localRDF.DS);
        el.builder.rebuild();

        el = document.getElementById("services-to-search");
        el.database.AddDataSource(jabberService.localRDF.DS);
        el.builder.rebuild();

        el = document.getElementById("this-user");
        el.database.AddDataSource(jabberService.localRDF.DS);
        el.builder.rebuild();

        this.observer = new DataSourceObserver();
        jabberService.rdf.DS.AddObserver(this.observer);

        el = document.getElementById("commands.menu.jim.showAll");
        
        document.getElementById('user-list').showAll =
            el.getAttribute("checked") == "true";

        jabberService.addObserver(this);
        jabberService.core.addObserver(this);
        jabberService.core.iq.auth.addObserver(this);
        jabberService.core.highlevel.users.addObserver(this);
        jabberService.core.highlevel.avatars.addObserver(this);
    },

    onUnload: function()
    {
        this.updateTaskbarIcon(false);
        jabberService.rdf.DS.RemoveObserver(this.observer);
        jabberService.clean();
    },

    onUsersChanged: function(users, userid, resourceid)
    {
        var resource, user, show;
        var win;

        if (!resourceid)
            return;

        resource = jabberService.core.highlevel.users.
            getInfoByResourceID(resourceid);
        user = jabberService.core.highlevel.users.
            getInfoByUserID(userid);

        if (resource.localizedName)
            alertMgr.showAlert(resource.localizedName, 
                resource.showTextual, resource.show, resource.status,
                jabberService.core.highlevel.avatars.
                    getAvatarByResourceId(resourceid));
        else
            alertMgr.showAlert(user.name, 
                this.bundle.GetStringFromName("userShowUnavailable"),
                "unavailable");
    },

    onAvatarChanged: function(users, userid, resourceid)
    {
        var win, resource;

        if (!resourceid)
            return;

        resource = jabberService.core.highlevel.users.
            getInfoByResourceID(resourceid);

        if (alertMgr.isOpened(resource.localizedName)) {
            alertMgr.updateAlert(null, null, null, null,
                jabberService.core.highlevel.avatars.
                    getAvatarByResourceId(resourceid));
        } else {
            this.onUsersChanged(users, userid, resourceid);
        }
    },

    onMessage: function(tag)
    {
        var win = this.__getChatWindow(tag.getAttribute('from'));
        if (win)
            win.focus();
        else
            openDialog("chrome://jim/content/chat.xul", 
                "_blank", "chrome,all,dialog=no",
                tag.getAttribute('from'), tag);
    },

    onIQAuth: function()
    {
        document.getElementById('main-deck').selectedIndex = 1;
    },

    onIQError: function(tag)
    {
        if (document.getElementById("main-deck").selectedIndex == 2) {
            var errTag = tag.getElementsByTagName("error")[0];

            document.getElementById("jim-main-error").value =
                this.bundle.formatStringFromName(
                    "error", [errTag.getAttribute("code"), domToText(errTag)], 2);

            document.getElementById("main-deck").selectedIndex = 3;
        }
    },

    onIQAuthError: function(errorCode, errorText)
    {
        document.getElementById("jim-main-error").value =
            this.bundle.formatStringFromName(
                "error", [errorCode, errorText], 2);

        document.getElementById("main-deck").selectedIndex = 3;
    },

    onConnect: function(status)
    {
        this.updateTaskbarIcon(true);
    },

    onDisconnect: function(status)
    {
        this.updateTaskbarIcon(false);

        if (document.getElementById("main-deck").selectedIndex == 2) {
            document.getElementById("jim-main-error").value =
                this.bundle.formatStringFromName(
                    "error", [status, statusCodeToString(status)], 2);
            document.getElementById("main-deck").selectedIndex = 3;
        } else
            document.getElementById('main-deck').selectedIndex = 0;
    },

    updateTaskbarIcon: function(value)
    {
        var wmediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].
            getService(Components.interfaces.nsIWindowMediator);
        var it, win, el;

        it = wmediator.getEnumerator(null);

        while (it.hasMoreElements()) {
            win = it.getNext();
            el = win.document.getElementById("jim-statubar-icon");
            if (el)
                el.setAttribute("connected", value ? "true":"false");
        }
    },

    __getWindow: function(windowType)
    {
        var wmediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].
            getService(Components.interfaces.nsIWindowMediator);

        return wmediator.getMostRecentWindow(windowType);
    },

    __getChatWindow: function(from)
    {
        var wmediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].
            getService(Components.interfaces.nsIWindowMediator);
        var it, win;

        it = wmediator.getEnumerator("jim:chat");

        while (it.hasMoreElements()) {
            win = it.getNext();
            if (win.contact == from)
                return win;
        }
        return null;
    },
};

function DataSourceObserver()
{
    this.batchMode = false;
}

DataSourceObserver.prototype =
{
    onAssert: function(dataSource, source, property, target)
    {
        if (!this.batchMode)
            mainHandler.profileChanged();
    },

    onUnassert: function(dataSource, source, property, target)
    {
        if (!this.batchMode)
            mainHandler.profileChanged();
    },

    onChange: function(dataSource, source, property, oldTarget, newTarget)
    {
        if (!this.batchMode)
            mainHandler.profileChanged();
    },

    onMove: function(dataSource, oldSource, newSource, property, target)
    {
        if (!this.batchMode)
            mainHandler.profileChanged();
    },

    beginUpdateBatch: function(datasource)
    {
        this.batchMode = true;
    },


    endUpdateBatch: function(dataSource)
    {
        this.batchMode = false;
        mainHandler.profileChanged();
    }

}

var mainHandler = new MainHandler();
